! id:netcdf_routines.f, 2018/5/25 xxu
!******************************************************************************
!  NETCDF_ROUTINES contains a list of routines that process netcdf I/O.
!  These routines were copied from other unl-vrtm modules in order to
!  seperate netcdf operations, so that a libray of unl-vrtm can be
!  compiled without calling the netcdf library.
!  (Comments added 9/27/2018, xxu)
!
!  ROUNTINES:
!  --------------------------------------------------------------------
!  (1) READ_GAS_OPTICS_OFFLINE: copied from gas_mod.f (5/25/2018)
!  (2) READ_ATMOS_PROF_NC:      copied from atmprof_mod.f (5/25/2018) 
!  (3) READ_GC_AEROSOL_PROF:    copied from atmprof_mod.f (5/25/2018) 
!
!------------------------------------------------------------------------------
!
      SUBROUTINE READ_GAS_OPTICS_OFFLINE

      USE ATMPROF_MOD,        ONLY : N_LAYERS
      USE ATMPROF_MOD,        ONLY : ATMOS
      USE NAMELIST_ARRAY_MOD, ONLY : FILE_TRACEGAS_OPTICS
      USE NAMELIST_ARRAY_MOD, ONLY : NGAS, LGAS, GASNAME
      USE NAMELIST_ARRAY_MOD, ONLY : LDEBUG_TRACEGAS_CALC
      USE NAMELIST_ARRAY_MOD, ONLY : LFREQUENCY
      USE NAMELIST_ARRAY_MOD, ONLY : LContinuum
      USE NAMELIST_ARRAY_MOD, ONLY : LHITRAN, LSAOX
      USE SPECTRA_MOD,        ONLY : NSPECTRA, WAVENUMS
      USE SPECTRA_MOD,        ONLY : LAMDAS
      USE ERROR_MOD,          ONLY : ERROR_STOP
      USE GAS_MOD,            ONLY : XSEC_SAO, XSEC_HIT, XSEC_CTN
      USE GAS_MOD,            ONLY : XABS_SAO, XABS_HIT, XABS_CTN
      USE GAS_MOD,            ONLY : GAS_XSEC, GAS_XABS
      USE GAS_MOD,            ONLY : DEBUG_GAS_OPTICS
      USE NETCDF_MOD

      IMPLICIT NONE

      ! LOcal variables
      INTEGER                     :: NCID
      INTEGER                     :: st3D(3), ct3D(3)
      CHARACTER(LEN=255)          :: ErrMsg
      INTEGER                     :: I, J, L
      REAL*8                      :: Molecules, LayerThickness
      REAL*8                      :: NewP(N_LAYERS)
      REAL*8                      :: NewV(N_LAYERS)
      REAL*8, ALLOCATABLE         :: OldP(:)
      REAL*8, ALLOCATABLE         :: OldV(:)

      ! Local Type
      TYPE PRECAL_GASOPT_TABLE
         INTEGER                  :: nLAYERS
         INTEGER                  :: NGAS
         INTEGER                  :: NSPECTRA
         REAL*4, ALLOCATABLE      :: SPECTRA(:)
         REAL*4, ALLOCATABLE      :: LAYT(:)
         REAL*4, ALLOCATABLE      :: LAYP(:)
         REAL*4, ALLOCATABLE      :: XSAO(:,:,:)
         REAL*4, ALLOCATABLE      :: XHIT(:,:,:)
         REAL*4, ALLOCATABLE      :: XCTN(:,:,:)
      ENDTYPE PRECAL_GASOPT_TABLE
      TYPE ( PRECAL_GASOPT_TABLE ):: TBL

      WRITE(6,100)
 100  FORMAT(/, 2X, '-GET_GAS_OPTICS_OFFLINE: Read gas Xsec table' )

      ! Open the file to read
      CALL netCDF_Open_Read( NCID, FILE_TRACEGAS_OPTICS )

      ! Get dimensions from netCDF file
      CALL netCDF_Get_Dim( NCID, 'Layer'  , TBL%nLAYERS  )
      CALL netCDF_Get_Dim( NCID, 'Spectra', TBL%NSPECTRA )
      CALL netCDF_Get_Dim( NCID, 'Gas'    , TBL%NGAS     )

      ! Allocate arrays to be read
      ALLOCATE ( TBL%SPECTRA( TBL%NSPECTRA ) )
      ALLOCATE ( TBL%LAYT( TBL%nLAYERS ) )
      ALLOCATE ( TBL%LAYP( TBL%nLAYERS ) )
      ALLOCATE ( TBL%XSAO( TBL%NSPECTRA, TBL%nLAYERS, TBL%NGAS ) )
      ALLOCATE ( TBL%XHIT( TBL%NSPECTRA, TBL%nLAYERS, TBL%NGAS ) )
      ALLOCATE ( TBL%XCTN( TBL%NSPECTRA, TBL%nLAYERS, TBL%NGAS ) )
      TBL%SPECTRA = 0.0
      TBL%LAYT    = 0.0
      TBL%LAYP    = 0.0
      TBL%XSAO    = 0.0
      TBL%XHIT    = 0.0
      TBL%XCTN    = 0.0
      st3D = (/1, 1, 1/)
      ct3D = (/TBL%NSPECTRA, TBL%nLAYERS, TBL%NGAS/)

      ! Read spectra
      IF ( LFREQUENCY ) THEN
      CALL netCDF_Read_Var( NCID, TBL%SPECTRA,'Wavenum',1,TBL%NSPECTRA )
      ELSE
      CALL netCDF_Read_Var( NCID, TBL%SPECTRA,'Lamdas',1,TBL%NSPECTRA )
      ENDIF

      ! Read pressure and temperature profile 
      CALL netCDF_Read_Var( NCID, TBL%LAYP, 'LayP', 1, TBL%nLAYERS )
      CALL netCDF_Read_Var( NCID, TBL%LAYT, 'LayT', 1, TBL%nLAYERS )

      ! Read cross-sections 
      IF ( LHITRAN ) THEN 
      CALL netCDF_Read_Var( NCID, TBL%XSAO, 'Xsec_SAO', st3D, ct3D )
      ENDIF

      IF (LSAOX) THEN 
      CALL netCDF_Read_Var( NCID, TBL%XHIT, 'Xsec_LBL', st3D, ct3D )
      ENDIF  

      IF ( LContinuum ) THEN
      CALL netCDF_Read_Var( NCID, TBL%XCTN, 'Xsec_CTN', st3D, ct3D )
      ENDIF

      ! Close file after reading
      CALL netCDF_Close( NCID )

      ! Check consistency between the simulation and the calculated
      ! gas optical Table read from the netCDF file
      ! (1 ) NGAS must be same
      ! (2 ) NSpectra must be same
      ! (3 ) Spectral wavelength/frequency must be same
      IF ( TBL%NSPECTRA /= NSPECTRA .OR. TBL%NGAS /= NGAS ) THEN

         WRITE(6,120) ' TBL%NSPECTRA = ', TBL%NSPECTRA
         WRITE(6,120) '     NSPECTRA = ', NSPECTRA
         WRITE(6,120) '     TBL%NGAS = ', TBL%NGAS
         WRITE(6,120) '         NGAS = ', NGAS
         ErrMsg = 'NSPECTRA or NGAS is not matched up ...'
         CALL ERROR_STOP( ErrMsg, 'gas_mod.f' )

      ENDIF

      IF ( LFREQUENCY ) THEN

        DO J = 1, nSpectra
           IF ( ABS( TBL%SPECTRA(J)-WAVENUMS(J) ) >= 1.0D-3 ) THEN
              WRITE(6,130) 'TBL%SPECTRA(J) = ', TBL%SPECTRA(J)
              WRITE(6,130) '   WAVENUMS(J) = ', WAVENUMS(J)
              WRITE(6,120) '         for J = ', J
              ErrMsg = ' TBL%SPECTRA(J) does not match WAVENUMS(J)'
              CALL ERROR_STOP( ErrMsg, 'gas_mod.f' )
           ENDIF
        ENDDO

      ELSE

        DO J = 1, nSpectra
           IF ( ABS( TBL%SPECTRA(J)-LAMDAS(J) ) >= 1.0D-3 ) THEN
              WRITE(6,130) 'TBL%WAVENUM(J) = ', TBL%SPECTRA(J)
              WRITE(6,130) '     LAMDAS(J) = ', LAMDAS(J)
              WRITE(6,120) '         for J = ', J
              ErrMsg = ' TBL%SPECTRA(J) does not match LAMDAS(J)'
              CALL ERROR_STOP( ErrMsg, 'gas_mod.f' )
           ENDIF
        ENDDO


      ENDIF

      ! Allocate arrays to be used for cross-section interpolation
      ALLOCATE ( OldP( TBL%nLAYERS ) )
      ALLOCATE ( OldV( TBL%nLAYERS ) )
      OldV = 0.0D0

      ! Reverse the layer pressures into ascending sequence
      OldP = TBL%LAYP(TBL%nLAYERS:1:-1)
      NewP = Atmos%Box%P(n_Layers:1:-1)

      ! Start a gas loop
      DO I = 1, NGAS

         ! Gas is not included
         IF ( .NOT. LGAS(I) ) THEN

            ! Absorption is none 
            XSEC_SAO(:,:,I) = 0d0
            XSEC_HIT(:,:,I) = 0d0
            XABS_SAO(:,:,I) = 0d0
            XABS_HIT(:,:,I) = 0d0

            WRITE(6,110) I, TRIM( GASNAME(I) )

         ! Otherwise included
         ELSE

            !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            ! TEMPORARY WORK
            ! THIS WILL BE UPDATED SOON >>>>>>>>>>>>>>>>>>>>>>>>>>>>
            ! - Interpolate TBL to XSEC >>>>>>>>>>>>>>>>>>>>>>>>>>>>
            ! ( Done on 11/19/2015, xxu )
            !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            !XSEC_SAO(:,:,I) = TBL%XSAO(:,:,I)
            !XSEC_HIT(:,:,I) = TBL%XHIT(:,:,I)
            !
            ! Interpolate TBL to XSEC
            DO J = 1, nSpectra

               ! Interpolate XSEC_SAO
               ! ------------------------------------------------------
               ! Now explicitly give the array size to subroutine
               ! INTERPOLATE_XSEC() to aviod segemetation fault.
               ! Thanks Chi Li (UC Berkeley) for verification.
               ! (bug fixed on 2/22/2019, xxu) 
               ! ------------------------------------------------------
               IF (LSAOX) THEN
                  OldV = TBL%XSAO(J,TBL%nLAYERS:1:-1,I)
                  CALL INTERPOLATE_XSEC( N_LAYERS, & 
                          OldP(1:N_LAYERS), OldV(1:N_LAYERS), &
                          TBL%nLAYERS, &
                          NewP(1:TBL%nLAYERS), NewV(1:TBL%nLAYERS) )
                  XSEC_SAO(J,1:n_Layers,I) = NewV(n_Layers:1:-1)
               ENDIF

               ! Interpolate XSEC_HIT
               IF ( LHITRAN ) THEN
                  OldV = TBL%XHIT(J,TBL%nLAYERS:1:-1,I)
                  CALL INTERPOLATE_XSEC(  N_LAYERS, & 
                          OldP(1:N_LAYERS), OldV(1:N_LAYERS), &
                          TBL%nLAYERS, &
                          NewP(1:TBL%nLAYERS), NewV(1:TBL%nLAYERS) )
                  XSEC_HIT(J,1:n_Layers,I) = NewV(n_Layers:1:-1)
               ENDIF

               ! Interpolate XSEC_CTN
               IF ( LContinuum ) THEN
                  OldV = TBL%XCTN(J,TBL%nLAYERS:1:-1,I)
               CALL INTERPOLATE_XSEC( N_LAYERS, &
                       OldP(1:N_LAYERS), OldV(1:N_LAYERS), &
                       TBL%nLAYERS, &
                       NewP(1:TBL%nLAYERS), NewV(1:TBL%nLAYERS) )
                  XSEC_CTN(J,1:n_Layers,I) = NewV(n_Layers:1:-1)
               ENDIF

            ENDDO

            ! Calculate optical depth from XSEC
            DO L = 1, N_LAYERS

               ! Gas density
               Molecules      = ATMOS%Box%AirDen(L) & 
                              * ATMOS%Box%MixRatio(L,I) * 1d-6
               LayerThickness = ATMOS%Box%Z(L) * 1d2

               ! From Hitran XSEC
               XABS_HIT(:,L,I) = Molecules  & 
                               * LayerThickness & 
                               * XSEC_HIT(:,L,I)

               ! From SAO XSEC
               IF ( TRIM( GASNAME(I) ) == 'O2' ) THEN
                  ! O2-O2 collison band 
                  XABS_SAO(:,L,I) = Molecules * Molecules & 
                                  * LayerThickness &
                                  * XSEC_SAO(:,L,I)
               ELSE
                  XABS_SAO(:,L,I) = Molecules &
                                  * LayerThickness & 
                                  * XSEC_SAO(:,L,I)
               ENDIF

               ! From Continuum XSEC
               IF ( LContinuum ) THEN
                  XABS_CTN(:,L,I) = Molecules & 
                                  * LayerThickness &
                                  * XSEC_CTN(:,L,I)
               ENDIF

            ENDDO ! L = 1, N_LAYERS

            ! Combine SAO and HIT
            !DO L = 1, N_LAYERS
            !DO J = 1, NSPECTRA
            ! 
            !   GAS_XSEC(J,L,I) = XSEC_HIT(J,L,I)+XSEC_SAO(J,L,I)
            !   GAS_XABS(J,L,I) = XABS_HIT(J,L,I)+XABS_SAO(J,L,I)
            !
            !ENDDO
            !ENDDO

            ! Update on 4/23/2017, xxu:
            GAS_XSEC(1:nSpectra,1:n_Layers,I) = & 
                    XSEC_HIT(1:nSpectra,1:n_Layers,I) &
                  + XSEC_SAO(1:nSpectra,1:n_Layers,I)
            GAS_XABS(1:nSpectra,1:n_Layers,I) = & 
                    XABS_HIT(1:nSpectra,1:n_Layers,I) &
                  + XABS_SAO(1:nSpectra,1:n_Layers,I)

            IF ( LContinuum ) THEN
               GAS_XABS(1:nSpectra,1:n_Layers,I) = & 
                    GAS_XABS(1:nSpectra,1:n_Layers,I) &
                  + XABS_CTN(1:nSpectra,1:n_Layers,I)
               GAS_XSEC(1:nSpectra,1:n_Layers,I) = & 
                    GAS_XSEC(1:nSpectra,1:n_Layers,I) &
                  + XSEC_CTN(1:nSpectra,1:n_Layers,I)
            ENDIF

         ENDIF  ! IF ( .NOT. LGAS(I) )

      ENDDO ! I = 1, NGAS

      ! Deallocate
      IF ( ALLOCATED( TBL%SPECTRA ) )  DEALLOCATE( TBL%SPECTRA )
      IF ( ALLOCATED( TBL%LAYT    ) )  DEALLOCATE( TBL%LAYT    )
      IF ( ALLOCATED( TBL%LAYP    ) )  DEALLOCATE( TBL%LAYP    )
      IF ( ALLOCATED( TBL%XSAO    ) )  DEALLOCATE( TBL%XSAO    )
      IF ( ALLOCATED( TBL%XHIT    ) )  DEALLOCATE( TBL%XHIT    )
      IF ( ALLOCATED( TBL%XCTN    ) )  DEALLOCATE( TBL%XCTN    )
      IF ( ALLOCATED( OldP        ) )  DEALLOCATE( OldP        )
      IF ( ALLOCATED( OldV        ) )  DEALLOCATE( OldV        )

      ! Screen print
      IF ( LDEBUG_TRACEGAS_CALC ) CALL DEBUG_GAS_OPTICS

      WRITE(6,'(2X,A)') & 
        '- GET_GAS_OPTICS_OFFLINE: done gas optics offline '
      WRITE(6,'(A)') REPEAT( '=', 79 )

 110  FORMAT( 4X,"F#",I2," XSEC for ", A4, " is NOT included!" )
 120  FORMAT( 4X, A, I6 )
 130  FORMAT( 4X, A, F15.6 )

      RETURN

      END SUBROUTINE READ_GAS_OPTICS_OFFLINE
!
!-------------------------------------------------------------------------------

      SUBROUTINE INTERPOLATE_XSEC( NOld, OldP, OldV, NNew, NewP, NewV )

      ! Added Nold and NNew to explicitly define the size of Old and New
      ! variables. 
      ! (xxu, 2/22/2019)

      USE ERROR_MOD,          ONLY : ERROR_STOP
      USE SPECTRA_MOD,        ONLY : SPLINE,     SPLINT

      IMPLICIT NONE

      ! Arguments
      INTEGER, INTENT(IN)   :: NOld, NNew
      REAL*8, INTENT(IN)    :: OldP(NOld)
      REAL*8, INTENT(IN)    :: OldV(NOld)
      REAL*8, INTENT(IN)    :: NewP(NNew)
      REAL*8, INTENT(INOUT) :: NewV(NNew)

      ! Local variables
      !INTEGER             :: N_Old,  N_New
      CHARACTER(LEN=255)  :: ErrMsg, ErrLoc
      REAL*8, ALLOCATABLE :: Y2(:)
      INTEGER             :: L

      ! get the array size & OldV and OldP must have same size
      !N_OLd = SIZE( OldP )
      !N_New = SIZE( NewP )

      IF ( SIZE(OldV) /= NOLd .OR. SIZE(NewV) /= NNew ) THEN

         ErrMsg = 'SIZE( OldP/NewP ) not equal to SIZE( OldV/NewV ), ' &
               // 'check inputs ... stopped... '
         ErrLoc = 'INTERPOLATE_XSEC (gas_mod.f)'
         WRITE(6,'(4X, "SIZE(OldP) =", I4)') NOLd
         WRITE(6,'(4X, "SIZE(NewP) =", I4)') NNew
         WRITE(6,'(4X, "SIZE(OldV) =", I4)') SIZE( OldV )
         WRITE(6,'(4X, "SIZE(NewV) =", I4)') SIZE( NewV )
         CALL ERROR_STOP( ErrMsg, ErrLoc )

      ENDIF

      ! Allocate necessary array
      ALLOCATE( Y2( NOld ) )

      ! Spline the old data sets
      CALL SPLINE( OldP, OldV, NOld, 0.0D0, 0.0D0, Y2 )

      DO L = 1, NNew

         IF ( NewP(L) < OldP(1) ) THEN
            NewV(L) = OldV(1)
         ELSE IF ( NewP(L) > OldP(NOld) ) THEN
            NewV(L) = OldV(NOld)
         ELSE
            CALL  SPLINT( OldP, OldV, Y2, NOld, NewP(L), NewV(L) )
            IF ( NewV(L) < 0.0D0 ) NewV(L) = 0.0D0
         ENDIF

      ENDDO

      ! Deallocate
      IF ( ALLOCATED( Y2 ) ) DEALLOCATE( Y2 )

      RETURN

      END SUBROUTINE INTERPOLATE_XSEC
!
!
!------------------------------------------------------------------------------

      SUBROUTINE READ_ATMOS_PROF_NC ( UserAtm )
!
!******************************************************************************
!
!  Read UserAtm from netCDF file:
!    - Grid%Z
!    - Grid%P
!    - Box%T
!    - Box%MixRatio (if provided)
! 
!  ( xxu, 11/18/2015 )
!******************************************************************************
!
      ! Referenced module
      USE NETCDF_MOD
      USE NAMELIST_ARRAY_MOD, ONLY : nGas,  GasName
      USE ATMPROF_MOD,        ONLY : ATMOS_PROFILE 
      USE ATMPROF_MOD,        ONLY : Atmos_TimeLoc, Atmos  

      IMPLICIT NONE

      ! Argument
      TYPE (ATMOS_PROFILE), INTENT(OUT)  :: UserAtm

      ! Local variables
      CHARACTER(LEN=255), PARAMETER  :: ncFile =  'atmos.nc'
      INTEGER                        :: ncid
      INTEGER                        :: nz
      INTEGER                        :: I

      ! Zerolize  arrays
      UserAtm%nLayers = 0
      UserAtm%nGas = 0
      UserAtm%Grid%Z = 0.0D0
      UserAtm%Grid%P = 0.0D0
      UserAtm%Grid%T = 0.0D0
      UserAtm%Grid%AirDen = 0.0D0
      UserAtm%Grid%MixRatio = 0.0D0
      UserAtm%Box%Z = 0.0D0
      UserAtm%Box%P = 0.0D0
      UserAtm%Box%T = 0.0D0
      UserAtm%Box%AirDen = 0.0D0
      UserAtm%Box%MixRatio = 0.0D0

      !================================================================
      ! REAL_USER_PROF_NCF begins here 
      !================================================================
      WRITE(6,100)
 100  FORMAT( 2X, '- READ_USER_PROF_NCF: read data from file atmos.nc')

      ! Open the file to read
      CALL netCDF_Open_Read( ncid, ncFile )

      ! Get dimensions from netCDF file
      CALL netCDF_Get_Dim( ncid, 'Layer', nz )
      UserAtm%nLayers = nz
      WRITE( 6, '(4X, "UserAtm%nLayer = ", I4)') nz

      ! Read height grid
      CALL netCDF_Read_Var( NCID, UserAtm%Grid%Z(0:nz), & 
                           'ZGrid', 1, nz+1 )
      WRITE( 6, 110) 'ZGrid'

      ! Read grid pressure
      CALL netCDF_Read_Var( ncid, UserAtm%Grid%P(0:nz), & 
                           'PGrid', 1, nz+1 )
      WRITE( 6, 110) 'PGrid'

      ! Read layer temperature
      CALL netCDF_Read_Var( ncid, UserAtm%Box%T(1:nz), & 
                           'T', 1, nz )
      WRITE( 6, 110) 'T'

      DO I = 1, nGas

         ! Read the gas mixing ratio if provided in the netCDF file
         IF ( netCDF_Var_Exist( ncid, TRIM( GasName(I) ) ) ) THEN

            CALL netCDF_Read_Var( ncid, UserAtm%Box%MixRatio(1:nz,I), &
                                  TRIM( GasName(I) ), 1, nz )

            ! Add up the UserAtm%nGas to indicate how many gas are read
            UserAtm%nGas = UserAtm%nGas + 1

            WRITE(6, 110) TRIM( GasName(I) )

         ENDIF

      ENDDO

      ! Read time and location
      IF ( netCDF_Var_Exist( ncid, 'NYMD' ) ) THEN
         CALL netCDF_Read_Var( ncid, Atmos_TimeLoc%nymd, 'NYMD' )
      ENDIF
      IF ( netCDF_Var_Exist( ncid, 'NHMS' ) ) THEN
         CALL netCDF_Read_Var( ncid, Atmos_TimeLoc%nhms, 'NHMS' )
      ENDIF
      IF ( netCDF_Var_Exist( ncid, 'Lat' ) ) THEN
         CALL netCDF_Read_Var( ncid, Atmos_TimeLoc%Lat, 'Lat' )
      ENDIF
      IF ( netCDF_Var_Exist( ncid, 'Lon' ) ) THEN
         CALL netCDF_Read_Var( ncid, Atmos_TimeLoc%Lon, 'Lon' )
      ENDIF

      ! Check the reasonability of the time & location
      Atmos_TimeLoc%TimeLoc_Available = & 
          (  Atmos_TimeLoc%nymd >= 19500000 .AND. & 
             Atmos_TimeLoc%nymd <= 22000000 .AND. &
             Atmos_TimeLoc%Lat  >= -90.0    .AND. &
             Atmos_TimeLoc%Lat  <=  90.0    .AND. &
             Atmos_TimeLoc%Lon  >= -180.0   .AND. & 
             Atmos_TimeLoc%Lat  <   180.0 )

      ! Close file after reading
      CALL netCDF_Close( ncid )

 110  FORMAT( 4X, 'Read ', A6 ' from atmos.nc'  )

      END SUBROUTINE READ_ATMOS_PROF_NC

!------------------------------------------------------------------------------

      SUBROUTINE READ_GC_AEROSOL_PROF( GCAer )

      ! Referenced module
      USE NETCDF_MOD
      USE ATMPROF_MOD, ONLY : GEOS_CHEM_AEROSOL
      USE ATMPROF_MOD, ONLY : Max_nAer
      USE ATMPROF_MOD, ONLY : Atmos

      IMPLICIT NONE

      ! Arguments
      TYPE (GEOS_CHEM_AEROSOL), INTENT(OUT) :: GCAer

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! This is a routine specifically for the CLARREO project:
      !   Here the will read the aerosol concentratons saved the in the 
      !   "atmos.nc" file for DST1, ..., DST4, and calculated the
      !   volumes.
      ! (xxu, 11/23/2015)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      ! Local variables
      CHARACTER(LEN=255), PARAMETER :: ncFile =  'atmos.nc'
      INTEGER                       :: ncid
      INTEGER                       :: nz
      INTEGER                       :: I, L, J
      REAL*8,  PARAMETER :: AV     = 6.022e23! AvogadroConstant[#/mol]
      CHARACTER(LEN=4)  , PARAMETER :: AerName(Max_nAer) = & 
          (/ 'DST1', 'DST2', 'DST3', 'DST4' /)
      REAL*8            , PARAMETER :: AerM(Max_nAer) = & 
          (/ 29.0D0, 29.0D0, 29.0D0, 29.0D0 /)  ! g/mol       
      REAL*8            , PARAMETER :: AerDen(Max_nAer) = & 
          (/ 2.5D0, 2.65D0, 2.65D0, 2.65D0  /)  ! g/cm^3

      !================================================================
      ! READ_GC_AER_PROFILE begins here 
      !================================================================
      WRITE(6,100)
 100  FORMAT( 2X,'- READ_GC_AER_PROFILE: read GC aerosol from atmos.nc')


      ! Open the file to read
      CALL netCDF_Open_Read( ncid, ncFile )

      ! Get dimensions from netCDF file
      CALL netCDF_Get_Dim( ncid, 'Layer', nz )
      GCAer%nLayer = nz
      WRITE( 6, '(4X, "GCAer%nLayer = ", I4)') nz

      ! Read data
      J = 0
      DO I = 1, Max_nAer

         ! Read the aerosol con (ppb) if provided in the netCDF file
         IF ( netCDF_Var_Exist( ncid, TRIM(AerName(I)) ) ) THEN

            ! Addup the counter 
            J = J + 1

            ! Read variable (ppb)
            GCAer%AerName(J) = TRIM(AerName(I))
            CALL netCDF_Read_Var( ncid, GCAer%Vol(1:nz,J), & 
                                  TRIM(AerName(I)), 1, nz )
            WRITE(6, 110) TRIM( AerName(I) )

            ! Calculate the vol
            DO L = 1, GCAer%nLayer

              !--------------------------------------------------------
              ! Units conversion for aerosol concentrations
              !  ppbv => um^3/um^2
              !  
              !         GCAer%Vol: ppbv
              !  Atmos%Box%AirDen: molec/cm^3
              !       Atmos%Box%Z: m 
              !              AerM: g/mol
              !            AerDen: g/cm^3
              !--------------------------------------------------------
              GCAer%Vol(L,J) =   &                        ! ( um^3/um^2 )
                               GCAer%Vol(L,J) * 1d-9    & ! => ppv
                             * Atmos%Box%AirDen(L) / AV & ! => mol/cm^3
                             * AerM(I)                  & ! => g/mol 
                             * Atmos%Box%Z(L)*1d2       & ! => cm
                             / AerDen(I)                & ! => g/cm^3
                             * 1d4             ! cm^3/cm^2 => um^3/um^2


            ENDDO

         ENDIF

      ENDDO

      ! Count the nAer
      GCAer%nAer = J

      ! Close file after reading
      CALL netCDF_Close( ncid )

 110  FORMAT( 4X, 'Read ', A6 ' from atmos.nc'  )

      END SUBROUTINE READ_GC_AEROSOL_PROF

